#ifndef _ASM_BITOPS_H
#define _ASM_BITOPS_H
/*
 * Copyright 1992, Linus Torvalds.
 *
 * This file is subject to the terms and conditions of the GNU General Public
 * License.  See the file README.legal in the main directory of this archive
 * for more details.
 */

/*
 * For the benefit of those who are trying to port Linux to another
 * architecture, here are some C-language equivalents.	You should
 * recode these in the native assmebly language, if at all possible.
 * To guarantee atomicity, these routines call cli() and sti() to
 * disable interrupts while they operate.  (You have to provide inline
 * routines to cli() and sti().)
 *
 * Also note, these routines assume that you have 32 bit integers.
 * You will have to change this if you are trying to port Linux to the
 * Alpha architecture or to a Cray.  :-)
 *
 * C language equivalents written by Theodore Ts'o, 9/26/92
 */

static inline int set_bit(int nr,void * vaddr)
{
	int	      mask, retval;
	unsigned long *addr = (unsigned long *)vaddr;
	unsigned long flags;
	
	addr += nr >> 5;
	mask = 1 << (nr & 0x1f);
	save_flags (flags);
	cli();
	retval = (mask & *addr) != 0;
	*addr |= mask;
	restore_flags(flags);
	return retval;
}

static inline int clear_bit(int nr, void * vaddr)
{
	int	mask, retval;
	unsigned long *addr = (unsigned long *)vaddr;
	unsigned long flags;

	addr += nr >> 5;
	mask = 1 << (nr & 0x1f);
	save_flags (flags);
	cli();
	retval = (mask & *addr) != 0;
	*addr &= ~mask;
	restore_flags (flags);
	return retval;
}

static inline int test_bit(int nr, void * vaddr)
{
	int	mask;
	unsigned long *addr = (unsigned long *)vaddr;

	addr += nr >> 5;
	mask = 1 << (nr & 0x1f);
	return ((mask & *addr) != 0);
}
#endif /* _ASM_BITOPS_H */
